@@ -336,16 +336,21 @@ def update_mutation_tables(data: PatientSampleData, idx, cols, hugo, table_size,
         maf_cols_value = list(set(cols) & set(maf_cols_options))
 
     hugo_symbols = maf_df['Hugo_Symbol'].unique()
+    hugo_value_in_maf = hugo if hugo is None else list(set(hugo) & set(hugo_symbols))
     variant_classifications = maf_df['Variant_Classification'].unique()
+    variant_in_maf = variant if variant is None else list(set(variant) & set(variant_classifications))
     cluster_assignments = [] if 'Cluster_Assignment' not in maf_df else maf_df['Cluster_Assignment'].unique()
+    cluster_in_maf = cluster if cluster is None else list(set(cluster) & set(cluster_assignments))
+    sample_options = maf_df['Sample_ID'].unique()
 
     filtered_maf_df = maf_df.copy()
-    if hugo:
-        filtered_maf_df = filtered_maf_df[filtered_maf_df['Hugo_Symbol'].isin(hugo)]
-    if variant:
-        filtered_maf_df = filtered_maf_df[filtered_maf_df['Variant_Classification'].isin(variant)]
-    if cluster and 'Cluster_Assignment' in maf_df:
-        filtered_maf_df = filtered_maf_df[filtered_maf_df['Cluster_Assignment'].isin(cluster)]
+    # Only filter by (previous) values if they exist in current maf
+    if hugo_value_in_maf:
+        filtered_maf_df = filtered_maf_df[filtered_maf_df['Hugo_Symbol'].isin(hugo_value_in_maf)]
+    if variant_in_maf:
+        filtered_maf_df = filtered_maf_df[filtered_maf_df['Variant_Classification'].isin(variant_in_maf)]
+    if cluster_in_maf:
+        filtered_maf_df = filtered_maf_df[filtered_maf_df['Cluster_Assignment'].isin(cluster_in_maf)]
 
     filtering_expressions = filter_query.split(' && ')
     for filter_part in filtering_expressions:
@@ -372,24 +377,34 @@ def update_mutation_tables(data: PatientSampleData, idx, cols, hugo, table_size,
             inplace=False
         )
 
-    sample_options = filtered_maf_df['Sample_ID'].unique()
-
-    filtered_maf_df = filtered_maf_df.dropna(axis=1, how='all')
+    # if filtered dataframe is not empty, remove columns with only NaNs
+    filtered_maf_empty = filtered_maf_df.shape[0] == 0
+    if not filtered_maf_empty:
+        filtered_maf_df = filtered_maf_df.dropna(axis=1, how='all')
 
     # pull all columns that differ between samples
     # use <= so we don't accidentally catch columns that have all NaNs for certain mutations (nunique == 0)
     columns_equivalent = filtered_maf_df.groupby('id').nunique().le(1).all()
-    sample_cols = columns_equivalent[~columns_equivalent].index.tolist()
 
-    # generate participant-level (cols w/ no difference between samples) and sample-level mafs
-    participant_maf = filtered_maf_df[~filtered_maf_df.index.duplicated(keep='first')].drop(columns=sample_cols)
-    participant_maf['id'] = participant_maf.index.tolist()
+    # generate sample-level dataframe (cols that differ between samples)
+    if filtered_maf_empty:
+        sample_cols = ['Sample_ID', 't_ref_count', 't_alt_count']  # if all mutations filtered out, pull out t_ref/alt_count columns for empty sample dataframe
+        sample_maf = pd.DataFrame(columns=sample_cols)
+        sample_maf.loc[:, 'Sample_ID'] = sample_options
+        sample_maf.index.name = 'id'
+    else:
+        sample_cols = columns_equivalent[~columns_equivalent].index.tolist()
+        sample_maf = filtered_maf_df[sample_cols]
 
     # todo change to sort samples by timing
-    sample_maf = filtered_maf_df[sample_cols].reset_index().sort_values(['id', 'Sample_ID']).set_index(
+    sample_maf = sample_maf.reset_index().sort_values(['id', 'Sample_ID']).set_index(
         ['id', 'Sample_ID']).unstack(1)
     sample_maf['id'] = sample_maf.index.tolist()
 
+    # generate participant-level (cols w/ no difference between samples)
+    participant_maf = filtered_maf_df[~filtered_maf_df.index.duplicated(keep='first')].drop(columns=sample_cols)
+    participant_maf['id'] = participant_maf.index.tolist()
+
     #####
 
     participant_table_data = participant_maf.iloc[page_current * table_size: (page_current + 1) * table_size]  #.to_dict('records')
